text1 = '''
# Стек через связный список
class Node:
    def __init__(self, data):
        self.data = data
        self.next = None

class Stack:
    def __init__(self):
        self.head = None

    def is_empty(self):
        return self.head is None

    def push(self, item):
        new_node = Node(item)
        new_node.next = self.head
        self.head = new_node

    def pop(self):
        if self.is_empty():
            return None
        else:
            popped_item = self.head.data
            self.head = self.head.next
            return popped_item

    def peek(self):
        if self.is_empty():
            return None
        else:
            return self.head.data

    def __str__(self):
        current = self.head
        stack_str = ""
        while current:
            stack_str += str(current.data) + " → "
            current = current.next
        return stack_str.rstrip(" → ")

# Очередь через связный список
class Node:
    def __init__(self, data):
        self.data = data
        self.next = None

class Queue:
    def __init__(self):
        self.head = None
        self.tail = None

    def is_empty(self):
        return not bool(self.head)

    def enqueue(self, data):
        new_node = Node(data)
        if not self.head:
            self.head = new_node
            self.tail = new_node
        else:
            self.tail.next = new_node
            self.tail = new_node

    def dequeue(self):
        data = self.head.data
        self.head = self.head.next
        if not self.head:
            self.tail = None
        return data

    def __len__(self):
        count = 0
        current = self.head
        while current:
            count += 1
            current = current.next
        return count

    def __str__(self):
        current = self.head
        queue_str = ""
        while current:
            queue_str += " → " + str(current.data)
            current = current.next
        return queue_str.lstrip(" → ")
'''+'abracadabrabibidi' +'''
# Двусвязнйы список
class Node:
    def __init__(self, data):
        self.data = data
        self.prev = None
        self.next = None

class DoublyLinkedList:
    def __init__(self):
        self.head = None

    def add_node(self, data):
        new_node = Node(data)
        if self.head is None:
            self.head = new_node
        else:
            current = self.head
            while current.next is not None:
                current = current.next
            current.next = new_node
            new_node.prev = current

    def delete_node(self, data):
        if self.head is None:
            return
        elif self.head.data == data:
            if self.head.next is not None:
                self.head = self.head.next
                self.head.prev = None
            else:
                self.head = None
        else:
            current = self.head
            while current.next is not None and current.next.data != data:
                current = current.next
            if current.next is None:
                return
            else:
                current.next = current.next.next
                if current.next is not None:
                    current.next.prev = current

    def __len__(self):
        count = 0
        current = self.head
        while current:
            count += 1
            current = current.next
        return count

    def __str__(self):
        if self.head == None:
            return f"Двусвязный список пустой"
        current = self.head
        dllist_str = ""
        while current:
            dllist_str += " ⇄ " + str(current.data)
            current = current.next
        return dllist_str.lstrip(" ⇄ ")
'''+'abracadabrabibidi' +'''
# Цикличный двусвязный список
class Node:
    def __init__(self, data=None):
        self.data = data
        self.prev = None
        self.next = None

class CircularDoublyLinkedList:
    def __init__(self):
        self.head = None
        self.tail = None

    def append(self, data):
        new_node = Node(data)
        if self.head is None:
            self.head = new_node
            self.tail = new_node
            new_node.prev = self.tail
            new_node.next = self.head
        else:
            new_node.prev = self.tail
            new_node.next = self.head
            self.tail.next = new_node
            self.head.prev = new_node
            self.tail = new_node

    def prepend(self, data):
        new_node = Node(data)
        if self.head is None:
            self.head = new_node
            self.tail = new_node
            new_node.prev = self.tail
            new_node.next = self.head
        else:
            new_node.prev = self.tail
            new_node.next = self.head
            self.head.prev = new_node
            self.tail.next = new_node
            self.head = new_node

    def delete(self, key):
        current_node = self.head
        while current_node:
            if current_node.data == key:
                if current_node == self.head:
                    self.head = current_node.next
                    self.tail.next = self.head
                    self.head.prev = self.tail
                elif current_node == self.tail:
                    self.tail = current_node.prev
                    self.head.prev = self.tail
                    self.tail.next = self.head
                else:
                    current_node.prev.next = current_node.next
                    current_node.next.prev = current_node.prev
                return
            current_node = current_node.next

    def __len__(self):
        count = 0
        current_node = self.head
        while current_node:
            count += 1
            current_node = current_node.next
            if current_node == self.head:
                break
        return count

    def __str__(self):
        cdllist_str = ""
        current_node = self.head
        while current_node:
            cdllist_str += str(current_node.data) + " ⇄ "
            current_node = current_node.next
            if current_node == self.head:
                break
        return " ⇄ " + cdllist_str
'''+'abracadabrabibidi' +'''
# Дерево
class Node:
    def __init__(self, value):
        self.value = value
        self.children = []

class Tree:
    def __init__(self):
        self.root = None

    def add_node(self, value, parent_value=None):
        node = Node(value)
        if parent_value is None:
            if self.root is not None:
                raise ValueError("У дерева уже есть корень")
            self.root = node
        else:
            parent_node = self.find_node(parent_value)
            if parent_node is None:
                raise ValueError("Родительский узел не найден")
            parent_node.children.append(node)

    def find_node(self, value):
        return self._find_node(value, self.root)

    def _find_node(self, value, node):
        if node is None:
            return None
        if node.value == value:
            return node
        for child in node.children:
            found = self._find_node(value, child)
            if found is not None:
                return found
        return None

    def __str__(self):
        return self._str_tree(self.root)

    def _str_tree(self, node, indent=0):
        result = "  " * indent + str(node.value) + "\n"
        for child in node.children:
            result += self._str_tree(child, indent + 2)
        return result
'''+'abracadabrabibidi' +'''
# Бинарное дерево
class Node:
    def __init__(self, data):
        self.data = data
        self.left = None
        self.right = None

class BinaryTree:
    def __init__(self):
        self.root = None

    def insert(self, data):
        new_node = Node(data)
        if self.root is None:
            self.root = new_node
        else:
            current = self.root
            while True:
                if data < current.data:
                    if current.left is None:
                        current.left = new_node
                        break
                    else:
                        current = current.left
                else:
                    if current.right is None:
                        current.right = new_node
                        break
                    else:
                        current = current.right

    def search(self, data):
        current = self.root
        while current is not None:
            if data == current.data:
                return True
            elif data < current.data:
                current = current.left
            else:
                current = current.right
        return False

    def delete(self, data):
        if self.root is not None:
            self.root = self._delete(data, self.root)

    def _delete(self, data, node):
        if node is None:
            return node

        if data < node.data:
            node.left = self._delete(data, node.left)
        elif data > node.data:
            node.right = self._delete(data, node.right)
        else:
            if node.left is None:
                return node.right
            elif node.right is None:
                return node.left

            temp = self._find_min_node(node.right)
            node.data = temp.data
            node.right = self._delete(temp.data, node.right)

        return node

    def _find_min_node(self, node):
        while node.left is not None:
            node = node.left
        return node

    def __str__(self):
        return '\n'.join(self._display(self.root)[0])

    def _display(self, node):
        if node.right is None and node.left is None:
            line = str(node.data)
            width = len(line)
            height = 1
            middle = width // 2
            return [line], width, height, middle

        if node.right is None:
            lines, n, p, x = self._display(node.left)
            s = str(node.data)
            u = len(s)
            first_line = (x + 1)*' ' + (n - x - 1)*'_' + s
            second_line = x*' ' + '/' + (n - x - 1 + u)*' '
            shifted_lines = [line + u*' ' for line in lines]
            return [first_line, second_line] + shifted_lines, n + u, p + 2, n + u // 2

        if node.left is None:
            lines, n, p, x = self._display(node.right)
            s = str(node.data)
            u = len(s)
            first_line = s + x*'_' + (n - x)*' '
            second_line = (u + x)*' ' + '\\' + (n - x - 1)*' '
            shifted_lines = [u*' ' + line for line in lines]
            return [first_line, second_line] + shifted_lines, n + u, p + 2, u // 2

        left, n, p, x = self._display(node.left)
        right, m, q, y = self._display(node.right)
        s = str(node.data)
        u = len(s)
        first_line = (x + 1)*' ' + (n - x - 1)*'_' + s + y*'_' + (m - y)*' '
        second_line = x*' ' + '/' + (n - x - 1 + u + y)*' ' + '\\' + (m - y - 1)*' '
        if p < q:
            left += [n*' ']*(q - p)
        elif q < p:
            right += [m*' ']*(p - q)
        zipped_lines = zip(left, right)
        lines = [first_line, second_line] + [a + u*' ' + b for a, b in zipped_lines]
        return lines, n + m + u, max(p, q) + 2, n + u // 2
'''+'abracadabrabibidi' +'''
# Хэш-таблица методом цеполк (когда данные с одинаковыми ключами хранятся в виде списка)
class HashTable:
    def __init__(self, size):
        self.size = size
        self.table = [[] for _ in range(self.size)]

    def hash_function(self, key):
        return hash(key) % self.size

    def insert(self, key, value):
        slot = self.hash_function(key)
        for pair in self.table[slot]:
            if pair[0] == key:
                pair[1] = value
                return
        self.table[slot].append([key, value])

    def find(self, key):
        slot = self.hash_function(key)
        for pair in self.table[slot]:
            if pair[0] == key:
                return pair[1]
        return None
'''+'abracadabrabibidi' +'''
# Хэш-таблица методом открытой адресации
class HashTable:
    def __init__(self, size):
        self.size = size
        self.table = [None] * size

    def hash_function(self, key):
        return hash(key) % self.size

    def insert(self, key, value):
        index = self.hash_function(key)
        while self.table[index]:
            if self.table[index][0] == key:
                break
            index = (index + 1) % self.size
        self.table[index] = (key, value)

    def find(self, key):
        index = self.hash_function(key)
        while self.table[index]:
            if self.table[index][0] == key:
                return self.table[index][1]
            index = (index + 1) % self.size
        return None
'''+'abracadabrabibidi' +'''
# Двоичная куча
class BinaryHeap():

    def __init__(self, type='max'):
        #type can be 'max' or 'min'
        
        self.type = type
        self.data = []
    
    
    def buildHeap(self, arr):
        data = arr[::]
        n = len(data)
        for i in range(n, -1, -1):
            data = self.heapify(data, n, i)
        
        self.data = data

    
    def heapify(self, arr, n, i):
        f = True if self.type=='max' else False
        extra = i
        l = 2 * i + 1
        r = 2 * i + 2

        if l < n and ((arr[i] < arr[l] and f) or (arr[i] > arr[l] and not f)):
            extra = l

        if r < n and ((arr[extra] < arr[r] and f) or (arr[extra] > arr[r] and not f)):
            extra = r

        if extra != i:
            arr[i], arr[extra] = arr[extra], arr[i]

            return self.heapify(arr, n, extra)
        else:
            return arr
    
    
    def insert(self, data):
        self.data.append(data)
        self.buildHeap(self.data)
        
    
    def sorted(self) -> list:
        data = self.data[::]
        n = len(data)

        for i in range(n-1, 0, -1):
            data[i], data[0] = data[0], data[i]
            data = self.heapify(data, i, 0)
            
        return data

    
    def del_root(self):
        root = self.data.pop(0)
        return root
''' + 'abracadabrabibidi' + '''
# Базовый декоратор
def decorator_name(func, decorator_args=None):
    def wrapper(*args, **kwargs): # Параметры, которые хотите передать в функцию func
        
        print('BEFORE') # То, что выполнится ДО функции
        rez = func(*args, **kwargs)
        print('AFTER') # То, что выполнится ПОСЛЕ функции
        
        return rez
        
    return wrapper
''' + 'abracadabrabibidi' + '''
# Очередь с приоритетом
class PriorityQueue:
    def __init__(self):
        self.heap = BinaryHeap()

    def enqueue(self, item, priority):
        self.heap.insert((priority, item))

    def dequeue(self):
        return self.heap.delMin()[1]

    def __str__(self):
        return self.heap.__str__()
''' + 'abracadabrabibidi' + '''
# Очередь через двусвязный список
class Node:
    def __init__(self, data):
        self.data = data
        self.next = None

class Queue:
    def __init__(self):
        self.head = None
        self.tail = None

    def is_empty(self):
        return not bool(self.head)

    def enqueue(self, data):
        new_node = Node(data)
        if not self.head:
            self.head = new_node
            self.tail = new_node
        else:
            self.tail.next = new_node
            self.tail = new_node

    def dequeue(self):
        data = self.head.data
        self.head = self.head.next
        if not self.head:
            self.tail = None
        return data

    def __len__(self):
        count = 0
        current = self.head
        while current:
            count += 1
            current = current.next
        return count

    def __str__(self):
        current = self.head
        queue_str = ""
        while current:
            queue_str += " → " + str(current.data)
            current = current.next
        return queue_str.lstrip(" → ")
'''



#######################################################################################################################
# Портфельный анализ с невырожденной ковариационной матрицей
#######################################################################################################################
def PAN_1(n,s_a,s_b, r_ab): # 
    """Инвестор сформировал портфель из акций компаний А и В, затратив на приобретение акций А в `n`
    раз больше средств чем на покупку акций В.
    Ожидаемая доходность за период владения акциями А и В составляет `e_a`% и `e_b`%,
    при этом стандартное отклонение доходности равно `s_a`% и `s_b`%, соответственно.
    Найдите (в %) стандартное отклонение доходности портфеля инвестора,
    если известно, что коэффициент корреляции доходностей акций А и В равен `r_ab`.

    Args:
        n (int): во сколько раз акций А больше, чем В
        s_a (numerical): стандартное отклонение доходности акций А в %
        s_b (numerical): стандартное отклонение доходности акций B в %
        r_ab (numerical): коэффициент корреляции доходностей акций А и В
        
    ## Prints
       `answer` C запятой вместо точки и сокращением до соответствующего количества знаков после запятой

    Returns:
        `answer` (tuple): Соответствующая величина
    """
    from sympy import symbols, Eq, solve
    
    a, b = symbols('a b')

    # Задание системы уравнений
    eq1 = Eq(a, n * b)
    eq2 = Eq(a + b, 1)
    
    # Решение системы
    solution = solve((eq1, eq2), (a, b))
    a, b = solution[a], solution[b]

    var_a = s_a**2
    var_b = s_b**2
    
    var_rx = b**2*var_b + (n*b)**2*var_a + 2*(n*b)*b *s_a*s_b*r_ab
    
    print('стандартное отклонение в % = ' + rrstr(var_rx**0.5,2))
    
    return [var_rx**0.5]
######################################################################################################################
def PAN_2(e_a, e_b, e_c,s_a,s_b,s_c):
    """Ожидаемая доходность и стандартное отклонение доходности
    за период для акций компаний А, В, С составляют 
    `e_a`% `e_b`% `e_c`% и `s_a`% `s_b`% `s_c`%, соответственно.
    - Предполагая независимость доходностей акций А, В и С, найдите (в %)
    ожидаемую доходность портфеля, составленного из этих акций так,
    чтобы дисперсия его доходности была минимальной.

    Args:
        e_a (numerical): Ожидаемая доходность A
        e_b (numerical): Ожидаемая доходность B
        e_c (numerical): Ожидаемая доходность C
        s_a (numerical): стандартное отклонение доходности A
        s_b (numerical): стандартное отклонение доходности B
        s_c (numerical): стандартное отклонение доходности C

    ## Prints
       `answer` C запятой вместо точки и сокращением до соответствующего количества знаков после запятой

    Returns:
        `answer` (tuple): Соответствующая величина
    """
    from sympy import symbols, solve, diff
    
    a,b = symbols('a b')
    
    f = a**2 * s_a**2 + b**2* s_b**2 + (1-a-b)**2 * s_c**2

    # Partial derivatives
    f_a = diff(f, a)
    f_b = diff(f, b)

    critical_points = solve([f_a, f_b], (a, b))
    
    a,b, c = critical_points[a],critical_points[b], 1 - critical_points[a] - critical_points[b]
    
    E = a*e_a + b*e_b + c*e_c
    
    print('Ожидаемая доходность портфеля в % = ' + rrstr(E, 2))
    
    return  [E]
######################################################################################################################
def PAN_3(e_a,e_b,s_a,s_b, r_ab):
    """Математическое ожидание доходности акций компаний А и В составляет `e_a`% и `e_b`%, 
    при этом стандартное отклонение доходности равно `s_a`% и `s_b`%, соответственно. Также известен 
    коэффициент корреляции доходностей акций А и В, `r_ab`. Найдите (короткие позиции допускаются):
    - доли акций А и В в портфеле с минимальной дисперсией доходности;
    - ожидаемую доходность и стандартное отклонение доходности такого портфеля.

    Args:
        e_a (numerical): Математическое ожидание доходности акции компаний А
        e_b (numerical): Математическое ожидание доходности акции компаний В
        s_a (numerical): Стандартное отклонение доходности акции компаний А
        s_b (numerical): Стандартное отклонение доходности акции компаний В 
        r_ab (numerical): Коэффициент корреляции доходностей акций А и В

    ## Prints
        `answer` каждое значение по очереди.<br>C запятой вместо точки и сокращением до соответствующего количества знаков после запятой

    Returns:
        `answer` (tuple): Соответствующие величины
    """


    muA, muB = e_a/100, e_b/100
    sigmaA, sigmaB = s_a/100, s_b/100

    roAB = r_ab

    a = (sigmaB**2 - roAB*sigmaA*sigmaB) / (sigmaA**2 + sigmaB**2 - 2*roAB*sigmaA*sigmaB)
    b = 1-a
    muR = a*muA + b*muB
    VarR = (a**2)*(sigmaA**2) + (b**2)*(sigmaB**2) + 2*a*b*roAB*sigmaA*sigmaB

    answer = (a,b,muR*100,(VarR**0.5)*100)
    
    print('Доля А = ' + rrstr(answer[0],2) + '; доля В = ' + rrstr(answer[1],2))
    print('Ожидаемая доходность = ' + rrstr(answer[2],1) + '; стандартное отклонение = ' + rrstr(answer[3],1))

    return answer
#######################################################################################################################
PAN = [PAN_1,PAN_2,PAN_3]


#######################################################################################################################
# Специальные дискретные случайные величины
#######################################################################################################################
def SDRV_1(na,nb,pa,pb):
    """Банк выдал кредиты двум группам заемщиков: `na` заемщиков в первой группе и `nb` – во второй.<br> 
    Известно, что заемщики из первой группы возвращают кредит с вероятностью `pa`,<br>
    а заемщики из второй группы – с вероятностью `pb`.<br>
    Пусть X – суммарное количество возвращенных кредитов для обеих групп.<br>
    Предполагая независимость заемщиков, найдите:<br>
    - стандартное отклонение X; 
    - асимметрию X. 

    Args:
        na (numerical): Количество заемщиков в 1-ой группе
        nb (numerical): Количество заемщиков в 2-ой группе
        pa (numerical): Вероятность возвращения кредита в 1-ой группе
        pb (numerical): Вероятность возвращения кредита в 2-ой группе

    ## Prints
        `answer` каждое значение по очереди.<br>C запятой вместо точки и сокращением до соответствующего количества знаков после запятой

    Returns:
        `answer` (tuple): Соответствующие величины
    """
    qa = 1-pa
    qb = 1-pb

    sigmaX = (na*pa*qa + nb*pb*qb)**0.5
    

    AsX = (na*pa*qa*(qa-pa) + nb*pb*qb*(qb-pb))/(sigmaX**3)
    
    answer = (sigmaX,AsX)
    
    print('std_X = ' + rrstr(sigmaX,1))
    print('As_X  = ' + rrstr(AsX,3))
    return answer
#######################################################################################################################
def SDRV_2(sigmaX,sigmaY,sigmaZ,s):
    """Независимые пуассоновские случайные величины X,Y,Z
    имеют следующие стандартные отклонения:
    - σX=`sigmaX`
    - σY=`sigmaY`
    - σZ=`sigmaZ`<br>
    Пусть S=X+Y+Z<br>
    Найдите:<br>
    - вероятность P(S=`s`)
    - наиболее вероятное значение суммы S
    - стандартное отклонение σS
    - асимметрию As(S)
    - эксцесс Ex(S)
    Args:
        sigmaX (numerical): Стандартное отклонение X
        sigmaY (numerical): Стандартное отклонение Y
        sigmaZ (numerical): Стандартное отклонение Z
        s (numerical): Значение S, вероятность которого нужно найти

    ## Prints
       `answer` каждое значение по очереди.<br>C запятой вместо точки и сокращением до соответствующего количества знаков после запятой

    Returns:
        `answer` (tuple): Соответствующие величины
    """
    import scipy.stats

    S =  scipy.stats.poisson(sigmaX**2 + sigmaY**2 + sigmaZ**2)
    
    AsS = (sigmaX**2 + sigmaY**2 + sigmaZ**2)/((S.var()**0.5)**3)
    
    numerator = (sigmaX**2 + sigmaY**2 + sigmaZ**2) + 3*(sigmaX**4 + sigmaY**4 + sigmaZ**4) + 6*(sigmaX**2 * sigmaY**2 + sigmaY**2 * sigmaZ**2 + sigmaZ**2 * sigmaX**2)
    denominator = (S.var()**0.5)**4
    ExS = (numerator/denominator) - 3
    
    answer = (S.pmf(s),S.median(),S.std(),AsS,ExS)
    
    print(f'P(S = {s}) = ' + rrstr(answer[0],3))
    print('Наиболее вероятное S = ' + str(answer[1]))
    print('Cтандартное отклонение σS = ' + rrstr(answer[2],3))
    print('асимметрия As(S) = ' + rrstr(answer[3],3))
    print(rrstr(answer[4],3),'эксцесс Ex(S)')

    return answer
#######################################################################################################################
def SDRV_3(N,n,G):
    """Монеты в количестве `N`<br>
    штук подбрасываются до тех пор, пока `n` раз<br>
    не выпадет `G` гербов.<br>
    Пусть X – число бросков до первого появления `G` гербов,<br>
    а Y – число бросков до последнего появления `G`гербов<br>
    (Y = общее число бросков). <br>
    Найдите:<br>
    - математическое ожидание X
    - стандартное отклонение X
    - коэффициент корреляции между X и Y
    - математическое ожидание XY

    Args:
        N (numerical): Общее количество монет
        n (numerical): Пока столько раз не выпадет `G` гербов
        G (numerical): Количество гербов

    ## Prints
       `answer` каждое значение по очереди.<br>C запятой вместо точки и сокращением до соответствующего количества знаков после запятой

    Returns:
        `answer` (tuple): Соответствующие величины
    """
    import scipy.stats
    import math
    def C(n,k):
        return math.comb(n,k)

    M = scipy.stats.geom(1/2 ** N * C(N,G))
    
    answer = (M.mean(),(M.var())**0.5,M.var()/(M.std() * (n*M.var())**0.5),M.var()+n*M.mean()**2)


    print(rrstr(answer[0],3),'Математическое ожидание X')
    print(rrstr(answer[1],3),'Стандартное отклонение X')
    print(rrstr(answer[2],3),'Коэффициент корреляции X и Y')
    print(rrstr(answer[3],1),'Математическое ожидание XY')

    return answer
#######################################################################################################################
def SDRV_4(N,r,b,n):
    """Корзина содержит `N` шаров,
    среди которых `r` – красных и `b` – синих.
    Из корзины, случайным образом, без возвращения извлекаются `n` шаров.
    Пусть X и Y обозначают количество красных и синих шаров среди извлеченных, соответственно. Найдите ковариацию Cov(X,Y).

    Args:
        N (numerical): Всего шаров в корзине
        r (numerical): Красных шаров в корзине
        b (numerical): Синих шаров в корзине
        n (numerical): Извлекается шаров без возвращения
    
    ## Prints
       `answer` C запятой вместо точки и сокращением до соответствующего количества знаков после запятой

    Returns:
        `answer` (tuple): Соответствующая величина
    """
    import math
    import scipy.stats

    Ex = r/N * n
    Ey = b/N * n
    Z = N - r - b

    def C(n,k):
        return math.comb(n,k)


    dXY = dict()
    for i in range(r+1):
        for j in range(b+1):
            for k in range(Z+1):
                if (i+j+k)==n:
                    p = C(r,i)*C(b,j)*C(Z,k)/C(N,n)
                    dXY.setdefault(i*j,0)
                    dXY[i*j]+=p

    val = [i for i in dXY.keys()]
    chn = [i for i in dXY.values()]
    
    Exy = scipy.stats.rv_discrete(values=(val,chn)).mean()
    Cov = Exy - Ex*Ey
    
    answer = tuple([Cov])
    
    print(rrstr(answer[0],2))
    
    return answer
#######################################################################################################################
SDRV = [SDRV_1,SDRV_2,SDRV_3,SDRV_4]


#######################################################################################################################
# Нормальные случайные векторы ДОПОЛНИТЕЛЬНЫЙ ПАКЕТ ФУНКЦИЙ
#######################################################################################################################
def ANRV_1(a,b,n):
    """Случайный вектор(X,Y) равномерно распределен в треугольнике x⩾0, y⩾0, `a`*x+y⩽`b`.
    Найдите математическое ожидание E(X^`n` * Y).

    Args:
        a (numerical): `a`*x+y⩽b
        b (numerical): a*x+y⩽`b`

    Returns:
        EX10Y: математическое ожидание E(X^`n` * Y)
    """
    import sympy
    
    x,y = sympy.symbols('x y',real=True)

    f = 1/sympy.integrate(1,(y,0,b-a*x),(x,0,1))

    EX10Y = sympy.integrate(f* x**n * y,(y,0,b-a*x),(x,0,1))
    EX10Y    

    return EX10Y
#######################################################################################################################
def ANRV_2(a,x_1,x_2,y_1,y_2,p):
    """Случайный вектор имеет плотность распределения f(x,y) =`a`*x + C*y , `x_1` < x < `x_2`, `y_1` < y < `y_2`,
    найдите константу C и вероятность P(X + Y > `p`)

    Args:
        a (numerical): Коэффициент при x
        x_1 (numerical): Левая граница значений x
        x_2 (numerical): Правая граница значений x
        y_1 (numerical): Левая граница значений y
        y_2 (numerical): Правая граница значений y
        p (numerical): P(X + Y > `p`)

    ## Prints
        `answer` каждое значение по очереди.<br>C запятой вместо точки и сокращением до 4 знаков после запятой

    Returns:
        `answer` (tuple): Соответствующие величины
    """
    
    import sympy
    x, y, C = sympy.symbols('x y C')
    
    f = a * x + C*y
    
    i = sympy.integrate(f, (y, y_1, y_2), (x,x_1, x_2))
    
    c0 = sympy.solve(i - 1)[0]
    f = f.subs(C, c0)

    answer = (c0,sympy.integrate(f, (y, -x+p, y_2), (x,x_1, x_2)))
    
    print('Константа C = ' + rrstr(answer[0],4))
    print(f'P(X+Y>{p}) = ' + rrstr(answer[0],4))
    
    return answer
#######################################################################################################################
def ANRV_3(C,a,b,p):
    """Случайный вектор (X,Y) имеет плотность распределения f(x,y)= `C`*e^(`a`*x+`b`*y),
    если 0 ⩽ x < +∞, 0 ⩽ y < +∞, 0, в остальных точках. Найдите вероятность P(X < `p`).    

    Args:
        C (numerical): Коэффициент при экспоненте
        a (numerical): Коэффициент при x
        b (numerical): Коэффициент при y
        p (numerical): P(X < `p`)

    Returns:
        answer(numerical): Вероятность P(X < `p`)
    """
    import sympy
    
    assert a*b==C, 'C должно быть равно a*b'
    
    x, y = sympy.symbols('x y')
    f = C*sympy.exp(a*x+b*y)
    
    answer = sympy.integrate(f,(y,0,'oo'),(x,0,p))
    
    return answer
#######################################################################################################################
def ANRV_4(c_x2 = 0,c_x = 0,c_xy = 0,c_y = 0,c_y2 = 0):
    """Случайный вектор (X,Y) имеет плотность распределения<br>
    f_X,Y(x,y) = ( [невлияющий коэф.] * e^-1/2*(c_x2 * x^2 + c_x * x + c_xy * xy + c_y * y + c_y2 * y^2 + [невлияющая константа]) / π
    
    Найдите:
    - математическое ожидание E(X)
    - математическое ожидание E(Y)
    - дисперсию  Var(X)
    - дисперсию  Var(Y)
    - ковариацию  Cov(X,Y)
    - коэффициент корреляции  ρ(X,Y)
    - математическое ожидание E(X|Y)
    - математическое ожидание E(Y|X)
    - дисперсию  Var(X|Y)
    - дисперсию  Var(Y|X)
    
    Args:
        c_x2 (numerical, optional): Коэффициент при x^2. По умолчанию 0.
        c_x (numerical, optional): Коэффициент при x. По умолчанию 0.
        c_xy (numerical, optional): Коэффициент при xy. По умолчанию 0.
        c_y (numerical, optional): Коэффициент при y. По умолчанию 0.
        c_y2 (numerical, optional): Коэффициент при y^2. По умолчанию 0.
        
    ## Prints
        `answer` каждое значение по очереди. Без округления

    Returns:
        answer (tuple): (`sol[EX]`, `sol[EY]`, `VarX`, `VarY`, `CovXY`, `roXY`,`EX_Y`,`EY_X`,`VarX_Y`,`VarY_X`)
    """

    import sympy
    #после выноса -1/2!!!!!!
    coefs = {
        'x^2': c_x2,
        'x': c_x,
        'xy': c_xy,
        'y': c_y,
        'y^2': c_y2,
    }
    C = sympy.Matrix([[coefs['x^2'], int(coefs['xy']/2)], [int(coefs['xy']/2), coefs['y^2']]])
    C1 = C**(-1)
    VarX = C1[0, 0]
    sigmaX = sympy.sqrt(VarX)
    VarY = C1[1, 1]
    sigmaY = sympy.sqrt(VarY)
    CovXY = C1[0, 1]
    roXY = CovXY/(sigmaX*sigmaY)
    EX, EY = sympy.symbols('EX, EY')
    equations = (
        sympy.Eq(int(coefs['x^2'])*EX + int(coefs['xy']/2)*EY, int(coefs['x']*(-1/2))),
        sympy.Eq(int(coefs['xy']/2)*EX + int(coefs['y^2'])*EY, int(coefs['y']*(-1/2)))
    )
    sol = sympy.solve(equations, (EX, EY))
    
    x,y = sympy.symbols('x y',real=True)

    EX_Y=sol[EX]+roXY*sigmaX/sigmaY*(y-sol[EY])
    EY_X=sol[EY]+roXY*sigmaY/sigmaX*(x-sol[EX])

    VarX_Y=sigmaX**2*(1-roXY**2)
    VarY_X=sigmaY**2*(1-roXY**2)
    print(f'EX = {sol[EX]}\nEY = {sol[EY]}\nVarX = {VarX}\nVarY = {VarY}\nCovXY = {CovXY}\nroXY = {roXY}\nEX_Y = {EX_Y}\nEY_X = {EY_X}\nVarX_Y = {VarX_Y}\nVarY_X = {VarY_X}')
    
    answer = (sol[EX],sol[EY],VarX,VarY,CovXY,roXY,EX_Y,EY_X,VarX_Y,VarY_X)

    return answer
#######################################################################################################################
ANRV=[ANRV_1,ANRV_2,ANRV_3,ANRV_4]


import numpy as np
from  inspect import getsource

def one_rrstr(x,n=0): # округление до n знаков после запятой
    if n == 0:
        return str(x)
    fmt = '{:.'+str(n)+'f}'
    return fmt.format(x).replace('.',',')

def rrstr(x,n):
    rrstr1 = np.vectorize(one_rrstr)
    res = rrstr1(x,n)
    if res.size ==1:
        return res[0]
    return res



#######################################################################################################################
# Выборки из конечной совокупности
#######################################################################################################################
def SFFP_1(Omega, n):
    """В группе Ω учатся `N` студентов, Ω={1,2,...,`N`}.
    Пусть X(i) – 100-балльная оценка студента i∈Ω.
    Из группы Ω случайным образом `n` раз выбирается студент ω∈Ω.
    Повторный выбор допускается. Пусть ωj – студент,
    полученный после выбора j=1,...,`n`, X(ωj)– его оценка.
    Среднюю оценку на случайной выборке обозначим X¯=1/`n` * ∑X(ωj).
    Оценки в группе даны: `Omega`. Требуется найти:
    - дисперсию Var(X¯)
    - центральный момент μ3(X¯)

    Args:
        Omega (str): Список оценок для студентов
        n (int): Размер выборки

    ## Prints
        `answer` каждое значение последовательно.<br>C запятой вместо точки и сокращенное до соответствующего количества десятичных знаков.

    Returns:
        `answer` (tuple): Соответствующие значения
    """
    import scipy.stats as st
    import numpy as np

    Omega = np.array(Omega.split(', ')).astype(int)

    Var = Omega.var(ddof=0)/n
    mu3 = st.moment(Omega, 3)/n**2
    answer = [Var, mu3]
    
    print(f'Дисперсия Var(X¯) = {rrstr(answer[0],3)}')
    print(f'Центральный момент μ3(X¯) = {rrstr(answer[1],3)}')
    
    return answer
#######################################################################################################################
def SFFP_2(Omega, n):
    """В группе Ω учатся `N` студентов, Ω={1,2,...,`N`}.
    Пусть X(i) – 100-балльная оценка студента i∈Ω.
    Из группы Ω случайным образом `n` раз выбирается студент ω∈Ω.
    Повторный выбор НЕ допускается. Пусть ωj – студент,
    полученный после выбора j=1,...,`n`, X(ωj)– его оценка.
    Среднюю оценку на случайной выборке обозначим X¯=1/`n` * ∑X(ωj).
    Оценки в группе даны: `Omega`. Требуется найти:
    - математическое ожидание E(X¯)
    - дисперсию Var(X¯)

    Args:
        Omega (str): Список оценок для студентов
        n (int): Размер выборки

    ## Prints
        `answer` каждое значение последовательно.<br>C запятой вместо точки и сокращенное до соответствующего количества десятичных знаков.

    Returns:
        `answer` (tuple): Соответствующие значения
    """
    import scipy.stats as st
    import numpy as np

    Omega = np.array(Omega.split(', ')).astype(int)

    mu = Omega.mean()
    Var = Omega.var()/n *  (Omega.size - n)/(Omega.size -1)

    answer = [mu,Var]
    
    print(f'математическое ожидание E(X¯) = {rrstr(answer[0],3)}')
    print(f'Дисперсия Var(X¯) = {rrstr(answer[1],3)}')
    
    
    return answer
#######################################################################################################################
def SFFP_3(marks, works, n_prep):
    """Распределение баллов на экзамене до перепроверки задано таблицей<br>
    `marks`<br>
    `works`<br>
    Работы будут перепроверять `n_prep` преподавателей, которые разделили
    все работы между собой поровну случайным образом.
    Пусть X¯ – средний балл (до перепроверки) работ,
    попавших к одному из преподавателей.
    Требуется найти:
    - математическое ожидание E(X¯);
    - стандартное отклонение σ(X¯).

    Args:
        marks (list): список оценок
        works (list): спиосок количеств работ
        n_prep (int): количество преподавателей

    ## Prints
        `answer` каждое значение последовательно.<br>C запятой вместо точки и сокращенное до соответствующего количества десятичных знаков.

    Returns:
        `answer` (tuple): Соответствующие значения
    """
    import numpy as np

    X = np.repeat(marks, works)
    mu = X.mean()
    s = X.std() * np.sqrt((X.shape[0] - X.shape[0]/n_prep)/(X.shape[0]/n_prep * (X.shape[0]-1)) )
    
    print('Математическое ожидание = ' + rrstr(mu,2))
    print('Стандартное отклонение = ' + rrstr(s,3))
    
    answer = [mu, s]
    
    return answer
#######################################################################################################################
def SFFP_4(text):
    """Две игральные кости, красная и синяя, подбрасываются до тех пор,
    пока не выпадет `n` различных (с учетом цвета) комбинаций очков.
    Пусть Ri – число очков на красной кости, а Bi – число очков на
    синей кости в комбинации с номером i.
    Случайные величины Xi задаются соотношениями: Xi=... ,i=1,...,`n`.
    Среднее арифметическое этих величин обозначим X¯=1/`n`∑Xi.
    Требуется найти: 
    - математическое ожидание E(X¯)
    - стандартное отклонение σ(X¯)
    
    Args:
        text (long-str): текст условия задачи
    
    ## Prints
        `answer` каждое значение последовательно.<br>C запятой вместо точки и сокращенное до соответствующего количества десятичных знаков.

    Returns:
        `answer` (tuple): Соответствующие значения
    """
    import re
    import numpy as np
    if not isinstance(text, str) or not text.strip():
        raise ValueError("Input text must be a non-empty string")
    text = text.replace("−", "-")
    try:
        coefficients = re.findall(r'([-+]?\d+)\s*R[iI]|([-+]?\d+)\s*B[iI]', text)
        coeff_Ri = int(coefficients[0][0]) if coefficients[0][0] else 0
        coeff_Bi = int(coefficients[1][1]) if coefficients[1][1] else 0
    except IndexError:
        raise ValueError("Failed to extract coefficients from text")

    try:
        match = re.findall(r'\b(\d+)\b\s+различных', text)
        n_combinations = int(match[0])
    except IndexError:
        raise ValueError("Failed to extract number of combinations from text")

    ribi = np.array([coeff_Ri, coeff_Bi])
    r_values = np.arange(1,7)
    e_r = r_values.sum()/r_values.size
    var_r = (r_values**2).sum()/r_values.size -e_r**2
    N = 6**2

    try:
        E_X = np.multiply(ribi, e_r).sum()
        s = np.sqrt(np.multiply(ribi**2, var_r).sum() / n_combinations * (N - n_combinations) / (N - 1))
    except (RuntimeError, ValueError) as e:
        raise RuntimeError("Numerical error: {}".format(e))

    answer = [E_X, s]

    print(rrstr(answer[0],1))
    print(rrstr(answer[1],3))
    
    return answer
#######################################################################################################################
def SFFP_5(n_coins, n_combinations):
    """Имеется `n_coins` пронумерованных монет.
    Монеты подбрасываются до тех пор, пока не выпадет `n_combinations` различных
    (с учетом номера монеты) комбинаций орел-решка.
    Пусть Xi – число орлов в комбинации с номером i;
    а X¯ = 1/`n_combinations` * ∑Xi – среднее число орлов в полученных таким образом комбинациях.
    Требуется найти:
    - математическое ожидание E(¯)
    - дисперсию Var(X¯)
    
    Args:
        n_coins (int): количество пронумерованных монет
        n_combinaations (int): количество различных комбинаций
        
    
    ## Prints
        `answer` каждое значение последовательно.<br>C запятой вместо точки и сокращенное до соответствующего количества десятичных знаков.

    Returns:
        `answer` (tuple): Соответствующие значения
    """
    import numpy as np
    import scipy.stats
    
    X = scipy.stats.binom(n_coins,1/2)
    N = 2**n_coins
    answer = [n_coins/2, X.var() /n_combinations * (N-n_combinations)/(N-1) ]
    
    print('Математическое ожидание = ' + rrstr(answer[0],1))
    print('Дисперсия = ' + rrstr(answer[1],3))
    
    return answer
#######################################################################################################################
def SFFP_6(X,Y,n_XY,n):
    """Эмпирическое распределение признаков X и Y
    на генеральной совокупности Ω={1,2,...,100} задано таблицей частот.
    Из Ω случайным образом без возвращения извлекаются 6 элементов. Пусть X¯
    и Y¯ – средние значения признаков на выбранных элементах.
    Требуется найти:
    - математическое ожидание E(X¯)
    - дисперсию Var(Y¯)
    - коэффициент корреляции ρ(X¯,Y¯)

    Args:
        X (list): Все значения, которые принимает X
        Y (list): Все значения, которые принимает Y
        n_XY (list): Значения таблицы частот двумерным списком
        n (int): количество элементов, извлекаемых без возвращения
        
    ## Prints
        `answer` каждое значение по очереди.<br>C запятой вместо точки и сокращением до соответствующего количества знаков после запятой

    Returns:
        `answer` (tuple): Соответствующие величины
    """
    from IPython.display import Math
    import numpy as np
    
    X = np.array(X)
    Y = np.array(Y)
    n_XY = np.array(n_XY)
    N = n_XY.sum()

    N = n_XY.sum()                  # Всего элементов в генеральной совокупности
    p_X = n_XY.sum(axis=1) / N  # частоты для X
    p_Y = n_XY.sum(axis=0) / N  # частоты для Y

    E_X = (X * p_X).sum()
    E_Y = (Y * p_Y).sum()

    # Найдем дисперсии Var(X) и Var(Y)
    Var_X = ((X) ** 2 * p_X).sum()  - E_X**2
    Var_Y = ((Y) ** 2 * p_Y).sum()  - E_Y**2
    Var_Y=np.dot(np.power(Y, 2), p_Y) - E_Y**2

    # Найдем ковариацию Cov(X, Y)
    cov_XY = 0
    for i in range(len(X)):
        for j in range(len(Y)):
            p_xy = n_XY[i, j] / N
            cov_XY += p_xy * (X[i] - E_X) * (Y[j] - E_Y)

    # Дисперсии выборочных средних
      # размер выборки
    Var_X_bar = Var_X /n * (N-n)/(N-1)
    Var_Y_bar = Var_Y /n * (N-n)/(N-1)

    # Ковариация выборочных средних
    Cov_XY_bar = cov_XY /n * (N-n)/(N-1)

    # Коэффициент корреляции 
    rho_XY = cov_XY / np.sqrt(Var_X * Var_Y)
    
    answer = [E_X, Var_Y_bar,rho_XY ]

    display(Math(('Математическое ожидание \mathbb{E}(\overline{X}) = ' + f'{rrstr(answer[0],3)}').replace(' ','~')))
    display(Math(('Дисперсия \mathbb{Var}(\overline{Y}) = ' + f'{rrstr(answer[1],3)}').replace(' ','~')))
    display(Math(('Коэффициент корреляции r(\overline{X},\overline{Y}) = ' + f'{rrstr(answer[2],3)}').replace(' ','~')))

    return answer
#######################################################################################################################
def SFFP_7(X,Y,n_XY,n):
    """Эмпирическое распределение признаков X и Y
    на генеральной совокупности Ω={1,2,...,100} задано таблицей частот.
    Из Ω случайным образом без возвращения извлекаются 6 элементов. Пусть X¯
    и Y¯ – средние значения признаков на выбранных элементах.
    Требуется найти:
    - математическое ожидание E(Y¯)
    - стандартное отклонение σ(X¯)
    - ковариацию Cov(X¯,Y¯)

    Args:
        X (list): Все значения, которые принимает X
        Y (list): Все значения, которые принимает Y
        n_XY (list): Значения таблицы частот двумерным списком
        n (int): количество элементов, извлекаемых без возвращения

    ## Prints
        `answer` каждое значение по очереди.<br>C запятой вместо точки и сокращением до соответствующего количества знаков после запятой

    Returns:
        `answer` (tuple): Соответствующие величины
    """
    from IPython.display import Math
    import numpy as np
    X = np.array(X)
    Y = np.array(Y)
    n_XY = np.array(n_XY)
    N = n_XY.sum()

    N = n_XY.sum()                  # Всего элементов в генеральной совокупности
    p_X = n_XY.sum(axis=1) / N  # частоты для X
    p_Y = n_XY.sum(axis=0) / N  # частоты для Y

    E_X = (X * p_X).sum()
    E_Y = (Y * p_Y).sum()

    # Найдем дисперсии Var(X) и Var(Y)
    Var_X = ((X - E_X) ** 2 * p_X).sum()
    Var_Y = ((Y - E_Y) ** 2 * p_Y).sum()

    # Найдем ковариацию Cov(X, Y)
    cov_XY = 0
    for i in range(len(X)):
        for j in range(len(Y)):
            p_xy = n_XY[i, j] / N
            cov_XY += p_xy * (X[i] - E_X) * (Y[j] - E_Y)

    # Дисперсии выборочных средних
    Var_X_bar = Var_X /n * (N-n)/(N-1)
    Var_Y_bar = Var_Y /n * (N-n)/(N-1)

    # Ковариация выборочных средних
    Cov_XY_bar = cov_XY /n * (N-n)/(N-1)

    # Коэффициент корреляции выборочных средних
    rho_XY_bar = Cov_XY_bar / (np.sqrt(Var_X_bar) * np.sqrt(Var_Y_bar))

    answer = [E_Y, Var_X_bar**0.5, Cov_XY_bar]
    
    display(Math(('Математическое ожидание \mathbb{E}(\overline{Y}) = ' + f'{rrstr(answer[0],3)}').replace(' ','~')))
    display(Math(('Стандартное отклонение \sigma(\overline{X}) = ' + f'{rrstr(answer[1],3)}').replace(' ','~')))
    display(Math(('Ковариация \mathbb{Cov}(\overline{X},\overline{Y}) = ' + f'{rrstr(answer[2],3)}').replace(' ','~')))

    return answer
#######################################################################################################################
SFFP = [SFFP_1,SFFP_2,SFFP_3,SFFP_4,SFFP_5,SFFP_6,SFFP_7]